SystemOrganization addCategory: #AttractorModelisation!

Object subclass: #Attractor
	instanceVariableNames: 'sigma rho beta dt values b attrView xValues yValues zValues graphList viewList ds precision xSteppedValues ySteppedValues zSteppedValues state initialState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!Attractor commentStamp: 'antoineZim 3/25/2017 16:28' prior: 0!
I am an empirical model of the Lorenz Attractor.

|a|
a := Attractor new.
a rebuild.
a!

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/25/2017 16:25'!
beta
	^beta! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/25/2017 16:25'!
beta: val
	beta := val! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 2/28/2017 23:37'!
buildAllViews 
	^(self buildAllViews: graphList)! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 2/28/2017 23:37'!
buildAllViews: graphCollection
	|newViewList|
	newViewList := OrderedCollection new.
	graphCollection do: [ :eachB | newViewList add: (self buildView: eachB) ].
	^newViewList! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 2/28/2017 23:20'!
buildRTGrapher

	b := RTGrapher new.
	ds := RTData new.
	ds dotShape ellipse size: 1; color: (Color blue).
	ds points: values.
	ds x: [:val | val at: 1]; y: [:val| val at: 2].
	b add: ds.
	b build.
	^b! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 2/28/2017 23:20'!
buildRTGrapher: valueList
	|ab ads|
	ab := RTGrapher new.
	graphList add: ab.
	ads := RTData new.
	ads dotShape ellipse size: 1.
	ads points: valueList .
	ads connectColor: Color red.
	"ads y: #yourself."
	ab add: ads.
	ab build.
	! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 2/28/2017 23:20'!
buildRTGrapher: anOrderedCollection withColor: color
	|ab ads|
	ab := RTGrapher new.
	ads := RTData new.
	ads dotShape ellipse size: 1; color: (Color blue).
	ads points: anOrderedCollection.
	ads connectColor: Color color.
	ads y: #yourself.
	ab add: ads.
	ab shouldUseNiceLabels: true.
	
	ab build.
	graphList add: ab! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 3/25/2017 16:33'!
buildValues
	| system stepper solver tempState |

	
	graphList :=OrderedCollection new.
	viewList :=OrderedCollection new.

	
	xValues := OrderedCollection new.
	yValues := OrderedCollection new.
	zValues := OrderedCollection new.
	state := initialState .
	system := ExplicitSystem
		block:[:val :t | |dx dy dz x y z|
			x := val at: 1. y := val at: 2. z := val at: 3.
			
			dx := sigma * (y - x).
			dy := beta * x - y - (x * z).
			dz := (-1 * rho * z + (x * y)).
			{dx. dy. dz}.
			].
	stepper := RungeKuttaStepper onSystem: system.
	solver := (ExplicitSolver new) stepper: stepper; system: system; dt: dt.
	tempState := state .
	values := (0.0 to: 100.0 by: dt) collect:[:t |
		tempState:= stepper doStep: tempState time:t stepSize: dt].
	values do: [ :val | 
						xValues add: (val at:1). 
						yValues add: (val at:2).
						zValues add: (val at:3)].
	xSteppedValues:= OrderedCollection new.
	ySteppedValues:= OrderedCollection new.
	zSteppedValues:= OrderedCollection new.
	xSteppedValues := xValues .
	ySteppedValues := yValues .
	zSteppedValues := zValues .
	"xSteppedValues addAllLast: xValues from: 1 to: xValues size  by: precision .
	ySteppedValues addAllLast: yValues from: 2 to: yValues size  by: precision .
	zSteppedValues addAllLast: zValues from: 3 to: zValues size  by: precision ."! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 2/28/2017 23:27'!
buildView: ab
	|aView anElementsAndEdges |
	aView := RTView new.
	aView @ RTDraggableView.

	anElementsAndEdges := ab view elements, ab view edges.
	aView addAll: anElementsAndEdges.

	
	aView canvas camera noInitializationWhenOpen.
	aView canvas camera scale: 1.5.

	^aView! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:20'!
ds
	^ds! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:21'!
dt
	^dt! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:21'!
dt: val
	dt:= val! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:21'!
elements
	^(b view elements)! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:21'!
graphList
	^graphList ! !

!Attractor methodsFor: 'initialization' stamp: 'antoineZim 3/24/2017 15:31'!
initialState
	^initialState ! !

!Attractor methodsFor: 'initialization' stamp: 'antoineZim 3/25/2017 16:29'!
initialize
	
	sigma := 10.0.
	beta := 30.0.
	rho := 8.0/3.0.
	dt := 0.1.
	precision := 25.
	initialState := #(10.0 10.0 10.0).
	self rebuild! !

!Attractor methodsFor: 'hooks' stamp: 'antoineZim 2/28/2017 23:21'!
openRTGrapher
	b open! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:21'!
precision
	^precision! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:21'!
precision: val
	precision := val! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 3/25/2017 16:29'!
rebuild
	self buildValues .
	"self halt."
	self buildRTGrapher: xSteppedValues .
	self buildRTGrapher: ySteppedValues .
	self buildRTGrapher: zSteppedValues .
	graphList add: (self buildRTGrapher).
	viewList := self buildAllViews! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/25/2017 16:25'!
rho
	^rho! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/25/2017 16:25'!
rho: val
	rho := val! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:22'!
sigma
	^sigma! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:22'!
sigma: val
	sigma := val! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/24/2017 14:21'!
state
	^state! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 3/24/2017 14:28'!
stateX: val
	state at:1 put: val! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 3/24/2017 14:28'!
stateY: val
	state at:2 put: val! !

!Attractor methodsFor: 'builders' stamp: 'antoineZim 3/24/2017 14:28'!
stateZ: val
	state at:3 put: val! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/24/2017 19:31'!
values
	^values! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:22'!
values: val
	values := val
! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:22'!
viewForGraphAt: indexInGraphList
	^((graphList at: indexInGraphList ) view)! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 2/28/2017 23:22'!
viewList
	^viewList 
	! !

!Attractor methodsFor: 'accessing' stamp: 'antoineZim 3/24/2017 17:48'!
xValues
	^xValues! !

Object subclass: #AttractorDynOdes
	instanceVariableNames: 'odes state data_length parameters initState sigma rho beta viewMaker xData yData zData dt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!AttractorDynOdes commentStamp: 'antoineZim 3/28/2017 20:53' prior: 0!
Make the reconstruction 
parameters := #(sigma,rho beta)
state := #(x y z)
odes := [:xyz  :sigmaRhoBeta | odesFunctions  ]

Try in playground: 
| attrDynOdes |
attrDynOdes := AttractorDynOdes new .
attrDynOdes rebuild .
attrDynOdes rk4 !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:58'!
beta
	^beta! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:57'!
beta: val
	beta := val.
	parameters at: 3 put: val
	! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/18/2017 16:46'!
dt: val 
	dt := val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/18/2017 16:50'!
initialize
	odes := self lorenz_odes.
	sigma := 10.0.
	rho := 8/3.0.
	beta := 28.0.
	dt := 0.02.
	viewMaker := ViewMaker2Dfrom3D new.
	initState := (Array with: -8.5 with: 8.0 with: 27.0 ).
	parameters :=(Array with: sigma with: rho with: beta).
	data_length := 1000.
	self rebuild! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/25/2017 16:54'!
lorenz_generate
	odes := self lorenz_odes .
	^(self rebuild)! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:15'!
lorenz_odes
	^(Array 	with: [:x :y :z :s :r :b | s * (y - x)  ] 
				with:[:x :y :z :s :r :b | b * x - y - (x * z)] 
				with: [:x :y :z :s :r :b | (-1 * r * z + (x * y))]).

! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 19:06'!
makeCompareGraphs
	| g g1 g2 g3 ds1 ds2 ds3 rec x y z |
rec := (AttractorDynOdes new) lorenz_generate.

x := (rec collect: [ : each| each at: 1 ]).
y := (rec collect: [ : each| each at: 2 ]).
z := (rec collect: [ : each| each at: 3 ]).
xData := x.
yData := y.
zData := z.
g := RTMultiGrapher new.

g1 := RTGrapher new.
ds1 := RTData new.
ds1 label: 'x values'.
ds1 connectColor: Color red.
ds1 points: x.
ds1 interaction highlightDataset: Color black.
g1 add: ds1.
g2 := RTGrapher new.
ds2 := RTData new.
ds2 label: 'y values'.
ds2 connectColor: Color blue.
ds2 points: y.
ds2 interaction highlightDataset: Color purple.
g2 add: ds2.

g3 := RTGrapher new.
ds3 := RTData new.
ds3 label: 'z values'.
ds3 connectColor: Color green.
ds3 points: z.
ds3 interaction highlightDataset: Color black.
g3 add: ds3.

g1 axisY
	title: 'y 1';
	color: Color red.
g2 axisY
	title: 'y 2';
	color: Color blue.
g3 	axisY
	title: 'y 3';
	color: Color green.
g add: g1.
g add: g2.
g add: g3.
g legend.
^ g! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/17/2017 14:18'!
odes
	^odes ! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/25/2017 17:06'!
odesX: val
	odes at:1 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/25/2017 17:06'!
odesY: val
	odes at:2 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/25/2017 17:06'!
odesZ: val
	odes at:3 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/24/2017 16:05'!
parameters
	^parameters ! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:54'!
parameters: val
	parameters :=val.
	sigma := parameters at:1.
	beta := parameters at: 2.
	rho := parameters at: 3! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:03'!
rebuild
	|data tempState x y z |
	tempState := state.
	data := self rk4.
	x := (data collect: [ : each| each at: 1 ]).
	y := (data collect: [ : each| each at: 2 ]).
	z := (data collect: [ : each| each at: 3 ]).
	xData := x.
	yData := y.
	zData := z.
	viewMaker values: data.
	viewMaker rebuild .
		
	^data  
	
! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 02:55'!
rebuildMainOnly
	|data tempState x y z |
	tempState := state.
	data := self rk4.
	x := (data collect: [ : each| each at: 1 ]).
	y := (data collect: [ : each| each at: 2 ]).
	z := (data collect: [ : each| each at: 3 ]).
	xData := x.
	yData := y.
	zData := z.
	viewMaker values: data.	
	viewMaker buildGraph.
	^data  
	
! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:13'!
rebuildSeriesOnly
	|data tempState x y z |
	tempState := state.
	data := self rk4.
	x := (data collect: [ : each| each at: 1 ]).
	y := (data collect: [ : each| each at: 2 ]).
	z := (data collect: [ : each| each at: 3 ]).
	xData := x.
	yData := y.
	zData := z.
	viewMaker values: data.	
	viewMaker buildSeriesGraphs .
	^data  
	
! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:58'!
rho
	^rho! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:58'!
rho: val
	rho := val.
	parameters at: 2 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/18/2017 16:49'!
rk4
	| system stepper solver s p r values tempState |
	s := parameters at:1 .
	p := parameters at:2.
	r := parameters at:3.
	state := initState.
	system := ExplicitSystem
		block:[:xyz :t | |dx dy dz x y z|
			x := xyz at: 1. y := xyz at: 2. z := xyz at: 3.
			dx := (odes at:1) valueWithArguments: (Array with:x with:y with:z with:s with:p with:r).
			dy := (odes at:2) valueWithArguments: (Array with:x with:y with:z with:s with:p with:r).
			dz := (odes at:3) valueWithArguments: (Array with:x with:y with:z with:s with:p with:r).
			{dx. dy. dz}.
			].
	stepper := RungeKuttaStepper onSystem: system.
	solver := (ExplicitSolver new) stepper: stepper; system: system; dt: dt.
	tempState := state .
	values := (80.0 to: 100.0 by: dt) collect:[:t |
		tempState:= stepper doStep: tempState time:t stepSize: dt].
	^values! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:58'!
sigma
	^sigma! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:57'!
sigma: val
	sigma := val.
	parameters at: 1 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/6/2017 18:52'!
stateX: val 
	state at: 1 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/6/2017 18:52'!
stateY: val 
	state at: 2 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/6/2017 18:52'!
stateZ: val 
	state at: 3 put: val! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:43'!
viewMaker
	^viewMaker ! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 19:07'!
xData
	^xData! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 19:07'!
yData
	^yData! !

!AttractorDynOdes methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 19:07'!
zData
	^zData! !

Object subclass: #AttractorDynOdesSecondVersion
	instanceVariableNames: 'odes data_length parameters initState sigma rho beta viewMaker xData yData zData dt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!AttractorDynOdesSecondVersion commentStamp: 'antoineZim 4/19/2017 05:36' prior: 0!
|attr|
attr := AttractorDynOdesSecondVersion new.
attr sigma: 10;
rho: 2;
beta:20;
dt:0.01;
stateX: 10;
stateY:10;
stateZ:10;
dataLength: 1000;
odesX: [:x :y :z :s :r :b | s * (y - x)  ];
odesY: [:x :y :z :s :r :b | b * x - y - (x * z)] ;
odesZ:[:x :y :z :s :r :b | (-1 * r * z + (x * y))].
attr rebuild .
attr viewMaker view!

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:27'!
allArgumentProvided
	^(((((((((((((sigma notNil  
	and: rho notNil) 
	and: beta notNil)
	and: dt notNil)
	and: (initState at: 1) notNil)
	and: (initState at: 2) notNil)
	and: (initState at: 3) notNil)
	and: (parameters at: 1) notNil)
	and: (parameters at: 2) notNil)
	and: (parameters at: 3) notNil)
	and: data_length notNil)
	and: (odes at: 1) notNil )
	and: (odes at: 2) notNil )
	and: (odes at: 3) notNil )
	! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:16'!
beta
	^beta! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:10'!
beta: val
	beta := val.
	parameters at: 3 put: val
	! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:29'!
dataLength:val
	data_length := val ! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:10'!
dt: val 
	dt := val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:24'!
initialize
	initState := Array with: nil with: nil with: nil.
	parameters := Array with: nil with: nil with: nil.
	odes := Array with: nil with: nil with: nil.
	viewMaker := ViewMaker2Dfrom3D new.
	
! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:16'!
lorenz_generate
	odes := self lorenz_odes .
	^(self rebuild)! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:33'!
lorenz_odes
	^(Array 	with: [:x :y :z :s :r :b | s * (y - x)  ] 
				with: [:x :y :z :s :r :b | b * x - y - (x * z)] 
				with: [:x :y :z :s :r :b | (-1 * r * z + (x * y))]).! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:16'!
odes
	^odes ! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:12'!
odesX: val
	odes at:1 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:13'!
odesY: val
	odes at:2 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:13'!
odesZ: val
	odes at:3 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:13'!
parameters: val
	parameters :=val.
	sigma := parameters at:1.
	beta := parameters at: 2.
	rho := parameters at: 3! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:14'!
rebuild
	|data tempState x y z |
	self allArgumentProvided ifTrue:[
	tempState := state.
	data := self rk4.
	x := (data collect: [ : each| each at: 1 ]).
	y := (data collect: [ : each| each at: 2 ]).
	z := (data collect: [ : each| each at: 3 ]).
	xData := x.
	yData := y.
	zData := z.
	viewMaker values: data.
	viewMaker rebuild .
		
	^data  ]
	
! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:14'!
rebuildMainOnly
	|data tempState x y z |
	self allArgumentProvided ifTrue:[
	tempState := state.
	data := self rk4.
	x := (data collect: [ : each| each at: 1 ]).
	y := (data collect: [ : each| each at: 2 ]).
	z := (data collect: [ : each| each at: 3 ]).
	xData := x.
	yData := y.
	zData := z.
	viewMaker values: data.	
	viewMaker buildGraph.
	^data  ]
	
! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:14'!
rebuildSeriesOnly
	|data tempState x y z |
	self allArgumentProvided ifTrue:[
	tempState := state.
	data := self rk4.
	x := (data collect: [ : each| each at: 1 ]).
	y := (data collect: [ : each| each at: 2 ]).
	z := (data collect: [ : each| each at: 3 ]).
	xData := x.
	yData := y.
	zData := z.
	viewMaker values: data.	
	viewMaker buildSeriesGraphs .
	^data  ]
	
! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:16'!
rho
	^rho! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:13'!
rho: val
	rho := val.
	parameters at: 2 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:12'!
rk4
	| system stepper solver s p r values tempState |
	s := parameters at:1 .
	p := parameters at:2.
	r := parameters at:3.
	system := ExplicitSystem
		block:[:xyz :t | |dx dy dz x y z|
			x := xyz at: 1. y := xyz at: 2. z := xyz at: 3.
			dx := (odes at:1) valueWithArguments: (Array with:x with:y with:z with:s with:p with:r).
			dy := (odes at:2) valueWithArguments: (Array with:x with:y with:z with:s with:p with:r).
			dz := (odes at:3) valueWithArguments: (Array with:x with:y with:z with:s with:p with:r).
			{dx. dy. dz}.
			].
	stepper := RungeKuttaStepper onSystem: system.
	solver := (ExplicitSolver new) stepper: stepper; system: system; dt: dt.
	tempState := initState .
	values := (80.0 to: 100.0 by: dt) collect:[:t |
		tempState:= stepper doStep: tempState time:t stepSize: dt].
	^values! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:16'!
sigma
	^sigma! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:14'!
sigma: val
	sigma := val.
	parameters at: 1 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:11'!
stateX: val 
	initState at: 1 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:11'!
stateY: val 
	initState at: 2 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:11'!
stateZ: val 
	initState at: 3 put: val! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:14'!
viewMaker
	^viewMaker ! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:14'!
xData
	^xData! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:14'!
yData
	^yData! !

!AttractorDynOdesSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:15'!
zData
	^zData! !

Object subclass: #Reconstructor
	instanceVariableNames: 'data_length values xValues yValues zValues xShifts yShifts zShifts tau flattenedValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!Reconstructor commentStamp: 'antoineZim 3/29/2017 14:18' prior: 0!
rec := Reconstructor new.
attrDynOdes := AttractorDynOdes new .
attrDynOdes rebuild .
rec values: attrDynOdes rk4.
rec rebuild.
v :=ViewMaker2Dfrom3D new .
v values: rec xShifts .
v rebuild .
v view.!

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 18:47'!
buildWithDataX: xCollection dataY: yCollection dataZ: zCollection
	xShifts := OrderedCollection new.
	yShifts := OrderedCollection new.
	zShifts := OrderedCollection new.
	xValues := xCollection.
	yValues := yCollection.
	zValues := zCollection.
	flattenedValues := OrderedCollection new.
	
	flattenedValues addAllLast: xValues .
	flattenedValues addAllLast: yValues .
	flattenedValues addAllLast: zValues .
	self buildXShiftedSeries .
	self buildYShiftedSeries .
	self buildZShiftedSeries ! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 22:24'!
buildXShiftedSeries
	| xShift0 xShift1 xShift2 dataLength|
	xShifts := OrderedCollection new.
	xShift0 := OrderedCollection new.
	xShift1 := OrderedCollection new.
	xShift2 := OrderedCollection new.
	dataLength := xValues size.

	flattenedValues 
			collect:[:each| xShift0 add:each]
				from: 1 to: dataLength.
	(Reconstructor new roll: flattenedValues shift: (-1*tau))
			collect:[:each| xShift1 add:each]
				from: 1 to: dataLength.
	(Reconstructor new roll: flattenedValues shift: (-2*tau))
			collect:[:each| xShift2 add:each]
				from: 1 to: dataLength.

	1 to: xValues size do: [ :i| 
		xShifts add: (Array 
			with: (xShift0 at:i) 
			with: (xShift1 at:i) 
			with: (xShift2 at:i)) ].
	^xShifts
	
! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 19:26'!
buildYShiftedSeries
	| tempShifts |
	yShifts := OrderedCollection new.
	tempShifts := OrderedCollection new.
	tempShifts add: yValues.
	tempShifts add: (self roll: yValues shift: -1*tau).
	tempShifts add: (self roll: yValues shift: -2*tau).
	1 to: yValues size do: [ :i| 
		yShifts add: (Array 
			with: ((tempShifts at:1) at:i) 
			with: ((tempShifts at:2) at:i) 
			with: ((tempShifts at:3) at:i)) ].
	^yShifts! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 19:27'!
buildZShiftedSeries
	| tempShifts |
	zShifts := OrderedCollection new.
	tempShifts := OrderedCollection new.
	tempShifts add: zValues.
	tempShifts add: (self roll: zValues shift: -1*tau).
	tempShifts add: (self roll: zValues shift: -2*tau).
	1 to: zValues size do: [ :i| 
		zShifts add: (Array 
			with: ((tempShifts at:1) at:i) 
			with: ((tempShifts at:2) at:i) 
			with: ((tempShifts at:3) at:i)) ].
	^zShifts! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 21:19'!
initialize
	tau := 5! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 20:44'!
rebuild
	self buildWithDataX: xValues  dataY: yValues  dataZ: zValues ! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/13/2017 16:40'!
roll: aCollection shift: value
	| newCollection val|
	val := value % (aCollection size).
	newCollection := OrderedCollection new.
	val>0 ifTrue:[
	1 to: (aCollection size) -val do: [:i| 
		 newCollection addLast: (aCollection at: i) ].
	aCollection size to: aCollection size - val+1 by:-1 do:[:i|
		newCollection addFirst: (aCollection at: i)]].
	val<=0 ifTrue:[
	aCollection size to:(val abs)+1 by:-1 do:[:i|
		newCollection addFirst: (aCollection at: i)].
	1 to: val abs  do: [:i| 
		 newCollection addLast:  (aCollection at: i) ].
	].
	^newCollection 
	! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 16:10'!
tau
	^tau! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/28/2017 21:02'!
tau: val
	tau := val! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 20:41'!
values: val

	xValues := OrderedCollection new.
	yValues := OrderedCollection new.
	zValues := OrderedCollection new.
	val do: [ :xyz| 
		xValues add: (xyz at:1).
		yValues add: (xyz at:2).
		zValues add: (xyz at:3) ].
	self buildWithDataX: xValues dataY: xValues  dataZ: xValues! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/28/2017 20:58'!
xShifts
	^xShifts ! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/28/2017 20:58'!
yShifts
	^yShifts ! !

!Reconstructor methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/28/2017 20:58'!
zShifts
	^zShifts ! !

Object subclass: #ViewMaker2Dfrom3D
	instanceVariableNames: 'values view graph dataset seriesGraphs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!ViewMaker2Dfrom3D commentStamp: 'antoineZim 3/29/2017 14:41' prior: 0!
Try in playground:
|v|
v :=ViewMaker2Dfrom3D new .
v values: AttractorDynOdes  new rk4.
v view!

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 22:29'!
buildGraph
	graph := RTGrapher new.
	dataset := RTData new.
	dataset noDot.
	dataset dotShape ellipse size: 1; color: (Color blue).
	dataset points: values.
	dataset x: [:val | val at: 1]; y: [:val| val at: 2].
	dataset connectColor: Color blue.
	graph add: dataset.
	graph build.
	RTCache new cacheView: graph view.
	^graph! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 22:40'!
buildSeriesGraphs
	| xValues yValues zValues dsX dsY dsZ |
	xValues := OrderedCollection new.
	yValues := OrderedCollection new.
	zValues := OrderedCollection new.
	values do: [ :val | 
						xValues add: (val at:1). 
						yValues add: (val at:2).
						zValues add: (val at:3)].
	seriesGraphs := OrderedCollection new.
	seriesGraphs add: RTGrapher new.
	seriesGraphs add: RTGrapher new.
	seriesGraphs add: RTGrapher new.
	
	dsX := RTData new.
	dsX dotShape ellipse size: 1; color: (Color blue).
	dsX points: xValues.
	dsX connectColor: Color red.
	dsX noDot.
	
	dsY := RTData new.
	dsY dotShape ellipse size: 1; color: (Color blue).
	dsY points: yValues.
	dsY connectColor: Color red.
	dsY noDot.

	dsZ := RTData new.
	dsZ dotShape ellipse size: 1; color: (Color blue).
	dsZ points: zValues.
	dsZ connectColor: Color red.
	dsZ noDot.	
		
	(seriesGraphs at:1) add: dsX.
	(seriesGraphs at:2) add: dsY.
	(seriesGraphs at:3) add: dsZ.
	seriesGraphs  do: [:g| g build.RTCache new cacheView: g view.].
	^seriesGraphs ! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/24/2017 18:38'!
graph
	^graph! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/25/2017 16:41'!
rebuild
	self buildGraph.
	self buildSeriesGraphs ! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:22'!
values: val
	values:= val.
	self rebuild! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/24/2017 19:28'!
view
	^(graph view)! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:49'!
viewX
	^((seriesGraphs at:1) view)! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:50'!
viewY
	^((seriesGraphs at:2) view)! !

!ViewMaker2Dfrom3D methodsFor: 'as yet unclassified' stamp: 'antoineZim 3/29/2017 14:50'!
viewZ
	^((seriesGraphs at:3) view)! !

DynamicComposableModel subclass: #UISecondVersion
	instanceVariableNames: 'mainView xtview ytview ztview plus slider varValues buildWithDataInputsButton attr precisionStep secondaryViews odesInputX odesInputY odesInputZ rec odesValues sizeOfUI sliderSigma sliderRho sliderBeta sliderX sliderY sliderZ recView v tau sliderTau reconstructionWidget dataInputWidget xData yData zData buildWithODESButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!UISecondVersion commentStamp: 'antoineZim 3/24/2017 19:01' prior: 0!
I am the new version

Try in playground:
|ui|
ui := UISecondVersion new.
ui refresh.
ui openView!

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/17/2017 18:12'!
buildWithDataInputsButton
	^buildWithDataInputsButton! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/17/2017 20:50'!
buildWithODESButton
	^buildWithODESButton ! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/17/2017 18:18'!
dataInputWidget
	^dataInputWidget! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/29/2017 14:30'!
initialize
	self initialize: AttractorDynOdes new! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/17/2017 19:27'!
initialize: attrIn
	"tau := 15."
	attr := attrIn.
	attr rebuild.
		xData := attr xData.
	yData := attr yData.
	zData := attr zData.
	"rec := Reconstructor new.
		rec tau: tau.
		rec values: attr rk4.
		rec rebuild.
	v :=ViewMaker2Dfrom3D new .
		v values: rec xShifts ."	
	super initialize.
	self initializeWidgets.
	self extent: 1400 @ 750.
			! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/19/2017 01:28'!
initializeWidgets

	self instantiateModels: #(
		mainView RoassalModel

		xtview RoassalModel
		ytview RoassalModel
		ztview RoassalModel
		sliderSigma SliderModel
		sliderRho SliderModel
		sliderBeta SliderModel
		sliderX SliderModel
		sliderY SliderModel
		sliderZ SliderModel
		varValues LabelModel
		buildWithDataInputsButton ButtonModel
		buildWithODESButton ButtonModel
		precisionStep TextInputFieldModel
		odesValue LabelModel
		odesInputX TextInputFieldModel
		odesInputY TextInputFieldModel
		odesInputZ TextInputFieldModel
		sizeOfUI LabelModel
		).
	layout := (SpecLayout composed
	newRow:[ :firstCol|
		firstCol 
		newColumn: [ :bCol | bCol 
				newRow: [ :r| r 
					newColumn:[:r1 | r1 
						add: #odesInputX ; 
						add: #odesInputY; 
						add: #odesInputZ; 
						add: #precisionStep;
						
						newRow:[:r2| r2 
							add: #buildWithDataInputsButton; 
							add: #buildWithODESButton]
						] ] height: 150;
				newRow: #dataInputWidget
				
				
				
				  ] width:250;
		newColumn: [ :col |
			col
				newRow: [:r | r add: #mainView; newColumn: [:c | c 
						add: #sliderSigma; 
						add: #sliderRho; 
						add: #sliderBeta;
						add: #sliderX ;
						add: #sliderY ; 
						add: #sliderZ]];
				newRow: [:c| c add: #reconstructionWidget];
				newRow:[:c | c 	add: #xtview; addSplitter;
									add: #ytview; addSplitter;
									add: #ztview];
				addSplitter;
				
				newRow: [:c | c add: #sizeOfUI; add: #varValues] height:25 ";
				newColumn: [:aCol| aCol
					newRow: [ :aRow | aRow  
					
					add: #precisionStep ] height: 25 ]"]
			
		];
	yourself).
	self title: 'Live-EDM_EasyRec'.
	dataInputWidget := DataInputWidget new.
	reconstructionWidget := ReconstructionWidget new.
	reconstructionWidget rebuildWithxData: xData   yData: yData  zData: zData.
	"Odes"
	buildWithODESButton := ButtonModel new.
	self buildWithODESButton label: 'Build reconstruction with ODES inputs'; state: false; 
			action: [
				xData := (attr xData).
				yData := (attr yData).
				zData := (attr zData).
				reconstructionWidget 
					rebuildWithxData: xData
					yData: yData
					zData: zData . 
				reconstructionWidget buildViewMakers.
				reconstructionWidget refresh ].
	
	odesInputX := TextInputFieldModel new.
	odesInputX beForCode.
	self odesInputX entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Odes [:x :y :z :sigma :rho :beta | your odes)';		
		whenTextChanged: [:text| | currentBlock isCompilable| 
			isCompilable := true.
			currentBlock := [(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0) ')] on: MessageNotUnderstood do: [isCompilable := false] . 
			 isCompilable ifTrue: [ (Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)  ') isNumber ifTrue: [ attr odesX:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,']')]. self refresh ]].
		
	odesInputY := TextInputFieldModel new.
	odesInputY beForCode.
	self odesInputY entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Odes [:x :y :z :sigma :rho :beta | your odes)';		
		whenTextChanged: [:text| | currentBlock isCompilable| 
			isCompilable := true.
			currentBlock := [(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0) ')] on: MessageNotUnderstood do: [isCompilable := false] . 
			 isCompilable ifTrue: [ (Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)  ') isNumber ifTrue: [ attr odesY:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,']')]. self refresh ]].
	
	odesInputZ := TextInputFieldModel new.
	odesInputZ beForCode.
	self odesInputZ entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Odes [:x :y :z :sigma :rho :beta | your odes)';		
		whenTextChanged: [:text| | currentBlock isCompilable| 
			isCompilable := true.
			currentBlock := [(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0) ')] on: MessageNotUnderstood do: [isCompilable := false] . 
			 isCompilable ifTrue: [ (Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)  ') isNumber ifTrue: [ attr odesZ:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,']')]. self refresh ]].
	
	odesValues := LabelModel new.
	self odesValues label:'odes= ',((attr odes) asString) .
	"Views"
	

	mainView := RoassalModel new.
	xtview := RoassalModel new.
	ytview := RoassalModel new.
	ztview := RoassalModel new.
		"recView := RoassalModel new."	
	"self recView script: [ :view :canvas |
		view addFromView: (v view).
		canvas camera focusOnCenter.
	]."
	"recView  := (v view canvas buildMorph) asSpecAdapter ."
	self mainView script: [ :view :canvas |
		view addFromView: (attr viewMaker view).
		canvas camera focusOnCenter.
	].
	self xtview script: [ :view :canvas |
		view addFromView: (attr viewMaker viewX) .
		canvas camera focusOnCenter.
	].
	self ytview script: [ :view :canvas |
		view addFromView: (attr viewMaker viewY).
		canvas camera focusOnCenter.
	].
	self ztview script: [ :view :canvas |
		view addFromView: (attr viewMaker viewZ).
		canvas camera focusOnCenter.
	].

	"Sliders"
	"sliderTau := SliderModel new."
	sliderSigma :=  SliderModel new.
	sliderRho :=  SliderModel new.
	sliderBeta :=  SliderModel new.
	sliderX :=  SliderModel new.
	sliderY :=  SliderModel new.
	sliderZ :=  SliderModel new.
	"sliderTau min:0;max:50;label:'0<=tau<=50'."
	sliderSigma "value: attr sigma;"min:0;max:20;label:'0<=sigma<=20'.
	sliderRho "value: attr rho ;"min:0;max:10;label:'0<=rho<=10';quantum: 0.2.
	sliderBeta "value: attr beta ;"min:0;max:40.0;label:'0<=beta<=40'.
	sliderX "value: (attr initialState at:1);" min:0;max:20;label:'0<=x<=10'.
	sliderY "value: (attr initialState at:2);" min:0;max:20;label:'0<=y<=10'.
	sliderZ "value: (attr initialState at:3);" min:0;max:20;label:'0<=z<=10'.
	
	"self sliderTau whenValueChangedDo: [:value |rec tau: value. self refresh]."
	
	self sliderSigma whenValueChangedDo: [:value |attr sigma: value. self refresh].
	self sliderRho whenValueChangedDo: [:value |attr rho: value. self refresh].
	self sliderBeta whenValueChangedDo: [:value |attr beta: value. self refresh].

	self sliderX whenValueChangedDo: [:value | attr stateX: value. self refresh].
	self sliderY whenValueChangedDo: [:value | attr stateY: value. self refresh].
	self sliderZ whenValueChangedDo: [:value | attr stateZ: value. self refresh].
					
	"feedback values"
	varValues := LabelModel new.
	self varValues label:'sigma=',(attr parameters at:1) asString ",'   r=',(attr r asString),'   precision=',(attr precision asString),'\n'",'   p =',(attr parameters at:2 )asString ,'   beta =',(attr parameters at:3) asString  .
	
	"Modifiers"
	precisionStep := TextInputFieldModel new.
	self precisionStep entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Precision (dt)';
		whenTextChanged: [:text| (text asInteger) isNil not ifTrue: [attr dt: (text asNumber)]. self refresh ].
	"DataInputs"
	buildWithDataInputsButton := ButtonModel new.
	self buildWithDataInputsButton label: 'Build reconstruction with dataInputs'; state: false; 
			action: [
				xData := (dataInputWidget dataCollection at:1).
				yData := (dataInputWidget dataCollection at:2).
				zData := (dataInputWidget dataCollection at:3).
				reconstructionWidget 
					rebuildWithxData: xData
					yData: yData
					zData: zData . 
				reconstructionWidget buildViewMakers.
				reconstructionWidget refresh ].

	
	
	"Testers"
	sizeOfUI := LabelModel new.
	self sizeOfUI label:'UIsize= ',((self sizeInMemory ) asString) .! !

!UISecondVersion methodsFor: 'accessing' stamp: 'antoineZim 2/23/2017 16:34'!
layout
	^layout! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 2/28/2017 11:44'!
mainView
	^mainView! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/25/2017 17:13'!
odesInputX
	^odesInputX! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/25/2017 17:13'!
odesInputY
	^odesInputY! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/25/2017 17:13'!
odesInputZ
	^odesInputZ! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 14:42'!
odesValues
	^odesValues! !

!UISecondVersion methodsFor: 'use' stamp: 'antoineZim 2/23/2017 16:40'!
openView
	
	self openWithSpecLayout: layout! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 2/27/2017 20:43'!
precisionStep
	^precisionStep! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/24/2017 18:54'!
recView
	^recView ! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/11/2017 13:05'!
reconstructionWidget
	^reconstructionWidget! !

!UISecondVersion methodsFor: 'use' stamp: 'antoineZim 4/18/2017 16:53'!
refresh
"TODO: find another solution to remake the model attr"
"attr refresh -> not working"
"attr view refresh -> not working"
	attr rebuild .
	xData := attr xData.
	yData := attr yData.
	zData := attr zData.
	reconstructionWidget 
					rebuildWithxData: xData
					yData: yData
					zData: zData .
	reconstructionWidget refresh.
	self varValues label:'sigma=',(attr sigma asString) ,'   rho =',(attr rho asString ),'   beta =',(attr beta asString ) ",'   r=',(attr r asString),'   precision=',(attr precision asString),'\n'".
	self odesValues label: 'odes= ',((attr odes) asString).
	self sizeOfUI label: 'sizeUI',((self sizeInMemory ) asString).
	self needRebuild: false.
	self buildWithSpecLayout: layout! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:10'!
sizeOfUI
	^sizeOfUI ! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:23'!
sliderBeta
	^sliderBeta ! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:23'!
sliderRho
	^sliderRho! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:23'!
sliderSigma
	^sliderSigma! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 4/6/2017 18:59'!
sliderTau
	^sliderTau! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:23'!
sliderX
	^sliderX! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:23'!
sliderY
	^sliderY! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 3/17/2017 17:24'!
sliderZ
	^sliderZ! !

!UISecondVersion methodsFor: 'use' stamp: 'antoineZim 2/27/2017 16:37'!
varValues
	^varValues! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 2/27/2017 16:39'!
xtview
	^xtview! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 2/27/2017 16:39'!
ytview
	^ytview! !

!UISecondVersion methodsFor: 'initialization' stamp: 'antoineZim 2/27/2017 16:38'!
ztview
	^ztview! !

DynamicComposableModel subclass: #UIThirdVersion
	instanceVariableNames: 'reconstructionWidget attractorWidget dataInputWidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!UIThirdVersion commentStamp: '<historical>' prior: 0!
I am the new version

Try in playground:
|ui|
ui := UIThirdVersion new.
ui refresh.
ui openView!

!UIThirdVersion class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:53'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
		newColumn: [:r| r
			newColumn: [ :row| row add: #attractorWidget ];
			addSplitter ;
			newColumn: [ :col| col 
				newRow: [ :row | row
					add: #reconstructionWidget 
					"newColumn: [ :col2 |col2
					add: #dataInputWidget] left:0 right: 0.67;
					newColumn: [ :col2 |col2
						add: #reconstructionWidget ] left:0.33 right: 0 "
				]  ]
			]! !

!UIThirdVersion class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:41'!
example
	"self example"

	UIThirdVersion   new openWithSpec.! !

!UIThirdVersion class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:42'!
title

	^ 'Live-EDM_EasyRec'! !

!UIThirdVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 17:03'!
attractor: attr
	reconstructionWidget attractor: attr.
	reconstructionWidget refresh ! !

!UIThirdVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:47'!
attractorWidget
	^attractorWidget ! !

!UIThirdVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:35'!
dataInputWidget
	^dataInputWidget! !

!UIThirdVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:27'!
initialize
	reconstructionWidget := ReconstructionWidgetSecondVersion new.
	attractorWidget := AttractorWidget new.
	attractorWidget parentWindow: self.
	dataInputWidget := DataInputWidget new.
	
	super initialize.
	

! !

!UIThirdVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:47'!
reconstructionWidget
	^reconstructionWidget! !

ComposableModel subclass: #AttractorWidget
	instanceVariableNames: 'odesInputX odesInputY odesInputZ attractor mainModel xModel yModel zModel sliderSigma sliderRho sliderBeta sliderX sliderY sliderZ precisionStep varValues displaySeries inputSigma inputRho inputBeta inputX inputY inputZ inputData_length parentWindow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!AttractorWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:51'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
	newRow:[ :firstCol|
		firstCol 
		newColumn: [ :col |
			col
			
				newRow: [:r | r 
					newColumn:[:r1 | r1 
						add: #odesInputX ; 
						add: #odesInputY; 
						add: #odesInputZ];
					add: #mainModel; 
					newColumn: [:c | c 
						add: #sliderSigma; 
						add: #sliderRho; 
						add: #sliderBeta;
						add: #sliderX ;
						add: #sliderY ; 
						add: #sliderZ;
						add: #precisionStep];
					newColumn: [ :col2|col2 
						add: #inputSigma;
						add: #inputRho;
						add: #inputBeta;
						add: #inputX;
						add: #inputY;
						add: #inputZ;
						add: #inputData_length
						]width: 80
					];addSplitter;
				newRow:[:r | r	add: #xModel; addSplitter;
									add: #yModel; addSplitter;
									add: #zModel];
				addSplitter;
				
					newRow: [:r | r add: #varValues;add: #displaySeries] height:25] 
				 	
		]! !

!AttractorWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 02:18'!
example
	"self example"

	AttractorWidget  new openWithSpec .! !

!AttractorWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 06:25'!
withoutSeriesModelsSpec
	<spec: #withoutSeriesModelsSpec>
	
	^ SpecLayout  composed
	newRow:[ :firstCol|
		firstCol 
		newColumn: [ :col |
			col
				newRow: [:r | r 
					newColumn:[:r1 | r1 
						add: #odesInputX ; 
						add: #odesInputY; 
						add: #odesInputZ];
					add: #mainModel; 
					newColumn: [:c | c 
						add: #sliderSigma; 
						add: #sliderRho; 
						add: #sliderBeta;
						add: #sliderX ;
						add: #sliderY ; 
						add: #sliderZ;
						add: #precisionStep
						];
							newColumn: [ :col2|col2 
								add: #inputSigma;
								add: #inputRho;
								add: #inputBeta;
								add: #inputX;
								add: #inputY;
								add: #inputZ;
								add: #inputData_length
						]width: 80
					];
				
				addSplitter;
				newRow: [:c | c add: #varValues;add: #displaySeries] height:25]	
		];yourself.! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:49'!
attractor
	^attractor ! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 02:28'!
displaySeries
	^displaySeries! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:43'!
initialize
	attractor := AttractorDynOdesSecondVersion new.
	super initialize.
	self initializeWidgets .


! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:48'!
initializeWidgets
	"Model views"
	mainModel := RoassalModel new.
	xModel := RoassalModel new.
	yModel := RoassalModel new.
	zModel := RoassalModel new.
	displaySeries := CheckBoxModel new.
	displaySeries state: true.
	displaySeries activationAction: [
		attractor notNil ifTrue: [attractor allArgumentProvided ifTrue: [attractor rebuildSeriesOnly]].
		self needRebuild: false.  
		self buildWithSpecLayout: 
		self class defaultSpec  ].
	displaySeries deactivationAction: [ 
		self needRebuild: false. 
		self buildWithSpecLayout: 
		self class withoutSeriesModelsSpec ].
	"displaySeries state -> boolean"
	"ODES Inputs"
	odesInputX := TextInputFieldModel new.
	odesInputY := TextInputFieldModel new.
	odesInputZ := TextInputFieldModel new.
	"Modifiers"
	inputData_length := TextInputFieldModel new.
	inputSigma := TextInputFieldModel new.
	inputRho := TextInputFieldModel new.
	inputBeta := TextInputFieldModel new.
	inputX := TextInputFieldModel new.
	inputY := TextInputFieldModel new.
	inputZ := TextInputFieldModel new.
	sliderSigma :=  SliderModel new.
	sliderRho :=  SliderModel new.
	sliderBeta :=  SliderModel new.
	sliderX :=  SliderModel new.
	sliderY :=  SliderModel new.
	sliderZ :=  SliderModel new.
	precisionStep := TextInputFieldModel new.
	self precisionStep entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Precision (dt)'.
	
	"Feedback"
	varValues := LabelModel new.
	
	self updateModifiers .! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:28'!
inputBeta
	^inputBeta! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 04:27'!
inputData_length
	^inputData_length! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:28'!
inputRho
	^inputRho! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:28'!
inputSigma
	^inputSigma! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:28'!
inputX
	^inputX! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:28'!
inputY
	^inputY! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 03:28'!
inputZ
	^inputZ! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:16'!
isAttractorBuildable
	^(attractor allArgumentProvided )! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:46'!
mainModel
	^mainModel! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:35'!
odesInputX
	^odesInputX! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:35'!
odesInputY
	^odesInputY! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:35'!
odesInputZ
	^odesInputZ! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:58'!
parentWindow: parent
	parentWindow := parent! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
precisionStep
	^precisionStep! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 17:10'!
refresh
	
	self needRebuild: false.
	self isAttractorBuildable ifTrue:[
		displaySeries state 
			ifTrue: [ attractor rebuild . 
				"self buildWithSpecLayout: self class defaultSpec" self updateModels  ] ;
		
			ifFalse:[attractor rebuildMainOnly. 
				"self buildWithSpecLayout: self class withoutSeriesModelsSpec"  self updateModels ].].
		self isAttractorBuildable ifTrue: [ parentWindow attractor: attractor ]

	! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
sliderBeta
	^sliderBeta! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
sliderRho
	^sliderRho! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
sliderSigma
	^sliderSigma! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
sliderX
	^sliderX! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
sliderY
	^sliderY! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:59'!
sliderZ
	^sliderZ! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 02:15'!
updateFeedbackLabels
	
	self varValues label:'sigma=',(attractor  parameters at:1) asString ",'   r=',(attr r asString),'   precision=',(attr precision asString),'\n'",'   p =',(attractor parameters at:2 )asString ,'   beta =',(attractor parameters at:3) asString  .! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 02:59'!
updateModels
	self mainModel script: [ :view :canvas |
		view addFromView: (attractor  viewMaker view).
		canvas camera focusOnCenter.
	].
	displaySeries state ifTrue: [ 
		self xModel script: [ :view :canvas |
			view addFromView: (attractor viewMaker viewX) .
			canvas camera focusOnCenter.
		].
		self yModel script: [ :view :canvas |
			view addFromView: (attractor viewMaker viewY).
			canvas camera focusOnCenter.
		].
		self zModel script: [ :view :canvas |
			view addFromView: (attractor viewMaker viewZ).
			canvas camera focusOnCenter.
		]. ]
	
	

	
	! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 06:14'!
updateModifiers
	"Sliders"
	

	sliderSigma "value: attractor sigma;"min:0;max:20;label:'0<=sigma<=20'.
	sliderRho "value: attractor rho ;"min:0;max:10;label:'0<=rho<=10';quantum: 0.2.
	sliderBeta "value: attractor beta ;"min:0;max:40.0;label:'0<=beta<=40'.
	sliderX "value: (attractor initialState at:1);" min:0;max:20;label:'0<=x<=10'.
	sliderY "value: (attractor initialState at:2);" min:0;max:20;label:'0<=y<=10'.
	sliderZ "value: (attractor initialState at:3);" min:0;max:20;label:'0<=z<=10'.
	
"	inputSigma beForCode.
	inputRho beForCode.
	inputBeta beForCode.
	inputX beForCode.
	inputY beForCode.
	inputZ beForCode."
	

	self inputData_length entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Data length';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor dataLength: (text asNumber) ]. self refresh].
	self inputSigma entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'inputSigma';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor sigma: (text asNumber) ]. self refresh].
	self inputRho entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'inputRho';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor rho: (text asNumber) ]. self refresh].
	self inputBeta entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'inputBeta';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor beta: (text asNumber) ]. self refresh].
	self sliderSigma whenValueChangedDo: [:value |attractor sigma: value. self refresh].
	self sliderRho whenValueChangedDo: [:value |attractor rho: value. self refresh].
	self sliderBeta whenValueChangedDo: [:value |attractor beta: value. self refresh].

	self inputX entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'inputX';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor stateX: (text asNumber) ]. self refresh].
	self inputY entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'inputY';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor stateY: (text asNumber) ]. self refresh].
	self inputZ entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'inputZ';		
		whenTextChanged: [:text|  (Compiler evaluate: '[',text,']') value isNumber ifTrue: [ attractor stateZ: (text asNumber) ]. self refresh].
	self sliderX whenValueChangedDo: [:value | attractor stateX: value. self refresh].
	self sliderY whenValueChangedDo: [:value | attractor stateY: value. self refresh].
	self sliderZ whenValueChangedDo: [:value | attractor stateZ: value. self refresh].
	
	"Others"
	self precisionStep entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Precision (dt)';
		whenTextChanged: [:text| (text asInteger) isNil not ifTrue: [attractor dt: (text asNumber)]. self refresh ].
		
	"ODES"
	odesInputX beForCode.
	self odesInputX entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Odes [:x :y :z :sigma :rho :beta | your odes)';		
		whenTextChanged: [:text| | currentBlock isCompilable| 
			isCompilable := true.
			currentBlock := [(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0) ')] on: MessageNotUnderstood do: [isCompilable := false] . 
			 isCompilable ifTrue: [ (Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)  ') isNumber ifTrue: [ attractor odesX:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,']')]. self refresh ]].
	
	odesInputY beForCode.
	self odesInputY entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Odes [:x :y :z :sigma :rho :beta | your odes)';		
		whenTextChanged: [:text| | currentBlock isCompilable| 
			isCompilable := true.
			currentBlock := [(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0) ')] on: MessageNotUnderstood do: [isCompilable := false] . 
			 isCompilable ifTrue: [ (Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)  ') isNumber ifTrue: [ attractor odesY:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,']')]. self refresh ]].
		
	odesInputZ beForCode.
	self odesInputZ entryCompletion: nil;
		acceptOnCR: true;
		ghostText: 'Odes [:x :y :z :sigma :rho :beta | your odes)';		
		whenTextChanged: [:text| | currentBlock isCompilable| 
			isCompilable := true.
			currentBlock := [(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0) ')] on: MessageNotUnderstood do: [isCompilable := false] . 
			 isCompilable ifTrue: [ (Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,'] valueWithArguments: (Array with:10.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)  ') isNumber ifTrue: [ attractor odesZ:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text,']')]. self refresh ]].! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 05:48'!
updateTools
	attractor allArgumentProvided 
	ifTrue: [ 
		self updateModels.
		self updateModifiers.
		self updateFeedbackLabels 
		]! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 02:00'!
varValues
	^varValues! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:47'!
xModel
	^xModel! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:47'!
yModel
	^yModel! !

!AttractorWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 01:47'!
zModel
	^zModel! !

ComposableModel subclass: #DataInputWidget
	instanceVariableNames: 'dataCollection clearButton bottom fileNameInput defaultLocation loadButton reconstructionParentWidget buildButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!DataInputWidget commentStamp: 'antoineZim 4/17/2017 18:05' prior: 0!
file must end with 
Character cr 
To fit the Reconstructor class, it is mendatory to load 3 datasets!

!DataInputWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 17:03'!
defaultSpec
	<spec>
	
	^ SpecLayout composed
		newColumn: [ :c| c newRow: [ :r| r add: #clearButton ; add: #buildButton ] height: 25;
		newRow: #fileNameInput height: 25;
		newRow: #bottom  ];
		
		yourself! !

!DataInputWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:27'!
example
	DataInputWidget   new openWithSpec .! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 15:54'!
bottom
	^bottom ! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:57'!
buildButton
	^buildButton! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 16:43'!
clearButton
	^clearButton! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 15:44'!
dataCollection
	^dataCollection ! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 15:41'!
fileNameInput
	^fileNameInput! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:55'!
initialize
	 
	dataCollection := OrderedCollection new.
	defaultLocation := (FileLocator imageDirectory)/'dataSets'.
	defaultLocation ensureCreateDirectory.
	super initialize.
! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 17:02'!
initializeWidgets
|txt|
	fileNameInput := self newTextInput .
	clearButton := self newButton.
	buildButton  := self newButton.
	bottom := self newList.
	
	self readDataFile: 'aze.txt'.
	self fileNameInput 
		acceptOnCR: true;
		ghostText: 'File name ...';		
		whenTextChanged: [:text| txt:= (text asString ).bottom  items: (self readDataFile: txt)   asOrderedCollection. ].
								
	clearButton 
		label: 'Clear';
		action: [ dataCollection := OrderedCollection new.
					bottom  items: dataCollection asOrderedCollection. 
			 ];
		state: false.
	buildButton 
		label: 'Build';
		action: [ reconstructionParentWidget rebuildWithxData: (dataCollection at:1) yData: (dataCollection at:2) zData: (dataCollection at:3) . 
			 ];
		state: false.
	bottom 
		items: dataCollection    asOrderedCollection.! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 18:02'!
readDataFile: name
	|file charCollection tempNum numCollection|
	numCollection := OrderedCollection new.
	charCollection := OrderedCollection new.
	file := (defaultLocation/name )asFileReference .
	file exists ifTrue:[
		file readStreamDo: [ :stream| charCollection addAll: stream contents ]].
	tempNum := ''.
	charCollection  
			collect:[:each|(each = Character cr  ) ifTrue: [ numCollection add: tempNum asNumber.tempNum := ''  ];ifFalse:
								[((each = Character lf  )or: [ each isNil ]) ifTrue: [ ]; ifFalse: [tempNum := tempNum,(each asString)]].
						]
				from: 1 to: charCollection size.
	dataCollection add: numCollection.
	^dataCollection 
		! !

!DataInputWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:56'!
reconstructionParentWidget: recWidget
	reconstructionParentWidget := recWidget! !

ComposableModel subclass: #ReconstructionWidget
	instanceVariableNames: 'buttonX buttonY buttonZ container morphX morphY morphZ sliderTau viewMakerX viewMakerY viewMakerZ reconstructor currentViewMaker labelTau recView currentButton viewMakerList attractor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!ReconstructionWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 16:12'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
		newColumn: [:r| r
			newRow: [ :row| row add: #recView ];
			newRow: [ :row| row add: #sliderTau ] height: 25;
			
			newRow: [ :row |
				row 
					add: #buttonX;
					add: #buttonY;
					add: #buttonZ;
					add: #labelTau ] height: 25
			
			]! !

!ReconstructionWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 13:02'!
example
	"self example"

	ReconstructionWidget  new openWithSpec.! !

!ReconstructionWidget class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 13:02'!
title

	^ 'Reconstruction Widget'! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 23:20'!
attractor: val
	attractor := val.
	reconstructor values: attractor rk4.
	self buildViewMakers .
	self refresh! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 16:36'!
buildViewMakers
	viewMakerX :=ViewMaker2Dfrom3D new .
		viewMakerX values: reconstructor xShifts .	
	viewMakerY :=ViewMaker2Dfrom3D new .
		viewMakerY values: reconstructor yShifts .	
	viewMakerZ :=ViewMaker2Dfrom3D new .
		viewMakerZ values: reconstructor zShifts .	
	viewMakerList := OrderedCollection new.
	viewMakerList add: viewMakerX ; add: viewMakerY ; add: viewMakerZ .! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 12:29'!
buttonX

	^ buttonX! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 12:29'!
buttonY

	^ buttonY! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 12:29'!
buttonZ

	^ buttonZ! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 15:53'!
initialize

	self initializeMorphs.
	super initialize.
	recView := RoassalModel new.
! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 23:14'!
initializeMorphs
	attractor := AttractorDynOdes new.
	reconstructor := Reconstructor new.
		reconstructor values: attractor rk4.

	currentViewMaker := 1.	
	self buildViewMakers.
! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 14:43'!
initializePresenter

	self on: buttonX show: viewMakerX.
	self on: buttonY show: viewMakerY.
	self on: buttonZ show: viewMakerZ.
	! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/13/2017 15:39'!
initializeWidgets

	buttonX := self newRadioButton.
	buttonY := self newRadioButton.
	buttonZ := self newRadioButton.
	labelTau := LabelModel new.
	sliderTau := SliderModel new.
	currentButton := buttonX .
	buttonX label: 'ButtonX'.
	buttonY label: 'ButtonY'.
	buttonZ label: 'ButtonZ'.
	sliderTau min:0;max:50;label:'0<=tau<=50'.
	
	self labelTau label:'tau= ',((reconstructor tau ) asString) .
	
	self sliderTau whenValueChangedDo: [:value |
		reconstructor tau: value. reconstructor rebuild. 
		self labelTau label:'tau= ',((reconstructor tau) asString). 	
		self buildViewMakers. 
		self refresh].	
	
	self setFocus.
	self setGroup.
	! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 16:10'!
labelTau
	^labelTau! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/13/2017 15:40'!
on: button show: aViewMaker

	button whenActivatedDo: [currentButton := button.
									((button label asString) = 'ButtonX') ifTrue: [currentViewMaker := 1] .
									((button label asString) = 'ButtonY') ifTrue: [currentViewMaker := 2] .
									((button label asString) = 'ButtonZ') ifTrue: [currentViewMaker := 3] .
									"currentViewMaker := viewMakerList indexOf: aViewMaker."  
									self refresh].
! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/17/2017 20:17'!
rebuildWithxData: xCollection yData: yCollection zData: zCollection
	reconstructor buildWithDataX: xCollection dataY: yCollection dataZ: zCollection.
	self buildViewMakers .
	self refresh! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 16:23'!
recView
	^recView ! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/15/2017 23:20'!
refresh
	
	self recView script: [ :view :canvas |
		view addFromView: (viewMakerList at: currentViewMaker)  view.
		canvas camera focusOnCenter.
	].! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 14:05'!
setFocus
	
	self focusOrder
		add: buttonX;
		add: buttonY;
		add: buttonZ. 
	! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 12:31'!
setGroup

	RadioButtonGroup new 
		addRadioButton: buttonX;
		addRadioButton: buttonY;
		addRadioButton: buttonZ;
		default: buttonX.
	! !

!ReconstructionWidget methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/11/2017 13:49'!
sliderTau
	^sliderTau! !

ComposableModel subclass: #ReconstructionWidgetSecondVersion
	instanceVariableNames: 'buttonX buttonY buttonZ container morphX morphY morphZ sliderTau viewMakerX viewMakerY viewMakerZ reconstructor currentViewMaker labelTau recView currentButton viewMakerList attractor dataInputWidget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!ReconstructionWidgetSecondVersion class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:54'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
	
	newRow: [ :r1 | r1
		newColumn: [:r| r
			newRow: [ :row| row add: #dataInputWidget ]
			
			];
		newColumn: [:r| r
			newRow: [ :row| row add: #recView ]
			
			];
			newColumn: [ :c | c
				newRow: [ :row| row add: #sliderTau ] height: 25;
				newColumn: [ :row | row 
					add: #buttonX;
					add: #buttonY;
					add: #buttonZ;
					add: #labelTau ] ] 
			
		]
		! !

!ReconstructionWidgetSecondVersion class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:31'!
example
	"self example"

	ReconstructionWidgetSecondVersion   new openWithSpec.! !

!ReconstructionWidgetSecondVersion class methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:26'!
title

	^ 'Reconstruction Widget'! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 08:10'!
attractor: val
	attractor := val.
	self rebuildWithxData: attractor xData  yData: attractor yData zData: attractor zData .
	self labelTau label:'tau= ',((reconstructor tau ) asString) .
	
	self sliderTau whenValueChangedDo: [:value |
		reconstructor tau: value. reconstructor rebuild. 
		self labelTau label:'tau= ',((reconstructor tau) asString). 	
		self buildViewMakers. 
		self refresh
	].	
! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:23'!
buildViewMakers
	viewMakerX :=ViewMaker2Dfrom3D new .
		viewMakerX values: reconstructor xShifts .	
	viewMakerY :=ViewMaker2Dfrom3D new .
		viewMakerY values: reconstructor yShifts .	
	viewMakerZ :=ViewMaker2Dfrom3D new .
		viewMakerZ values: reconstructor zShifts .	
	viewMakerList := OrderedCollection new.
	viewMakerList add: viewMakerX ; add: viewMakerY ; add: viewMakerZ .! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:23'!
buttonX

	^ buttonX! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:23'!
buttonY

	^ buttonY! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:23'!
buttonZ

	^ buttonZ! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 16:55'!
dataInputWidget
	^dataInputWidget! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/21/2017 17:02'!
initialize
	dataInputWidget := DataInputWidget new.
	dataInputWidget reconstructionParentWidget: self.
	currentViewMaker := 1.
	reconstructor := Reconstructor new.
	viewMakerList := OrderedCollection new.
	viewMakerList addAll: #(nil nil nil).
	recView := RoassalModel new.
	super initialize.

! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 08:07'!
initializePresenter

	self on: buttonX show: (viewMakerList at:1).
	self on: buttonY show: (viewMakerList at:2).
	self on: buttonZ show: (viewMakerList at:3).! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/20/2017 16:45'!
initializeWidgets
	buttonX := self newRadioButton.
	buttonY := self newRadioButton.
	buttonZ := self newRadioButton.
	labelTau := LabelModel new.
	sliderTau := SliderModel new.
	currentButton := buttonX .
	buttonX label: 'ButtonX'.
	buttonY label: 'ButtonY'.
	buttonZ label: 'ButtonZ'.
	sliderTau min:0;max:50;label:'0<=tau<=50'.
	self setFocus.
	self setGroup.
	! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:37'!
isRecDisplayable
	^(((viewMakerList at:1) notNil 
	and:(viewMakerList at:1) notNil )
	and: (viewMakerList at:1) notNil )! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
labelTau
	^labelTau! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
on: button show: aViewMaker

	button whenActivatedDo: [currentButton := button.
									((button label asString) = 'ButtonX') ifTrue: [currentViewMaker := 1] .
									((button label asString) = 'ButtonY') ifTrue: [currentViewMaker := 2] .
									((button label asString) = 'ButtonZ') ifTrue: [currentViewMaker := 3] .
									"currentViewMaker := viewMakerList indexOf: aViewMaker."  
									self refresh].
! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
rebuildWithxData: xCollection yData: yCollection zData: zCollection
	reconstructor buildWithDataX: xCollection dataY: yCollection dataZ: zCollection.
	self buildViewMakers .
	self refresh! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
recView
	^recView ! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:34'!
refresh
	self isRecDisplayable ifTrue:[
		self recView script: [ :view :canvas |
			view addFromView: (viewMakerList at: currentViewMaker)  view.
			canvas camera focusOnCenter.
		].
	]! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
setFocus
	
	self focusOrder
		add: buttonX;
		add: buttonY;
		add: buttonZ. 
	! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
setGroup

	RadioButtonGroup new 
		addRadioButton: buttonX;
		addRadioButton: buttonY;
		addRadioButton: buttonZ;
		default: buttonX.
	! !

!ReconstructionWidgetSecondVersion methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/19/2017 07:25'!
sliderTau
	^sliderTau! !

TestCase subclass: #AttractorDynOdesTest
	instanceVariableNames: 'attrDynOdes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AttractorModelisation'!

!AttractorDynOdesTest methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/3/2017 12:17'!
setUp
	attrDynOdes := AttractorDynOdes new .
	attrDynOdes rebuild .! !

!AttractorDynOdesTest methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/3/2017 12:34'!
testBuildGraph
	|  text1 |
	text1 := 'x+2'.
	attrDynOdes odesX:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text1,']').
	attrDynOdes rebuild.

	
! !

!AttractorDynOdesTest methodsFor: 'as yet unclassified' stamp: 'antoineZim 4/3/2017 12:34'!
testChangeOdes
	|  text1 |
	text1 := 'x+2'.
	attrDynOdes odesX:(Compiler evaluate: '[:x :y :z :sigma :rho :beta |',text1,']').
	self assert: (attrDynOdes odes at:1) argumentCount  = 6.
	self assert: ((attrDynOdes odes at:1) valueWithArguments: (Array with:3.0 with:10.0 with:10.0 with:10.0 with:30.0 with:8.0/3.0)) = 5.

	
! !
